@@ -1,4 +1,5 @@
-# Copyright 2009-2012 Yelp and Contributors
+# -*- coding: utf-8 -*-
+# Copyright 2009-2013 Yelp and Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,32 +23,24 @@ from optparse import SUPPRESS_USAGE
 from mrjob.runner import CLEANUP_CHOICES
 
 
-def add_protocol_opts(opt_group, protocol_choices, default_output_protocol):
-    """Add options related to choosing protocols. These are all deprecated
-    except for :opt:`--strict-protocols` and must be made available as
-    passthrough options so the job picks it up under Hadoop Streaming.
+def _append_to_conf_paths(option, opt_str, value, parser):
+    """conf_paths is None by default, but --no-conf or --conf-path should make
+    it a list.
+    """
+
+    if parser.values.conf_paths is None:
+        parser.values.conf_paths = []
+
+    # this method is also called during generate_passthrough_arguments
+    # the check below is to ensure that conf_paths are not duplicated
+    if value not in parser.values.conf_paths:
+        parser.values.conf_paths.append(value)
+
+
+def add_protocol_opts(opt_group):
+    """Add options related to choosing protocols.
     """
     return [
-        opt_group.add_option(
-            '--input-protocol', dest='input_protocol',
-            default=None, choices=protocol_choices,
-            help=('DEPRECATED: protocol to read input with (default:'
-                  ' raw_value)')),
-
-        opt_group.add_option(
-            '--output-protocol', dest='output_protocol',
-            default=default_output_protocol,
-            choices=protocol_choices,
-             help='DEPRECATED: protocol for final output (default: %s)' % (
-            'same as --protocol' if default_output_protocol is None
-            else '%default')),
-
-        opt_group.add_option(
-            '-p', '--protocol', dest='protocol',
-            default=None, choices=protocol_choices,
-            help=('DEPRECATED: output protocol for mappers/reducers. Choices:'
-                  ' %s (default: json)' % ', '.join(protocol_choices))),
-
         opt_group.add_option(
             '--strict-protocols', dest='strict_protocols', default=None,
             action='store_true', help='If something violates an input/output '
@@ -55,7 +48,32 @@ def add_protocol_opts(opt_group, protocol_choices, default_output_protocol):
     ]
 
 
-def add_runner_opts(opt_group):
+def add_basic_opts(opt_group):
+    """Options for all command line tools"""
+
+    return [
+        opt_group.add_option(
+            '-c', '--conf-path', dest='conf_paths', action='callback',
+            callback=_append_to_conf_paths, default=None, nargs=1,
+            type='string',
+            help='Path to alternate mrjob.conf file to read from'),
+
+        opt_group.add_option(
+            '--no-conf', dest='conf_paths', action='store_const', const=[],
+            help="Don't load mrjob.conf even if it's available"),
+
+        opt_group.add_option(
+            '-q', '--quiet', dest='quiet', default=None,
+            action='store_true',
+            help="Don't print anything to stderr"),
+
+        opt_group.add_option(
+            '-v', '--verbose', dest='verbose', default=None,
+            action='store_true', help='print more messages to stderr'),
+    ]
+
+
+def add_runner_opts(opt_group, default_runner='local'):
     """Options for all runners."""
     return [
         opt_group.add_option(
@@ -72,10 +90,6 @@ def add_runner_opts(opt_group):
                   " --no-bootstrap-mrjob if you've already installed mrjob on"
                   " your Hadoop cluster.")),
 
-        opt_group.add_option(
-            '-c', '--conf-path', dest='conf_path', default=None,
-            help='Path to alternate mrjob.conf file to read from'),
-
         opt_group.add_option(
             '--cleanup', dest='cleanup', default=None,
             help=('Comma-separated list of which directories to delete when'
@@ -100,6 +114,10 @@ def add_runner_opts(opt_group):
             help=('Copy file to the working directory of this script. You can'
                   ' use --file multiple times.')),
 
+        opt_group.add_option(
+            '--interpreter', dest='interpreter', default=None,
+            help=("Interpreter to run your script, e.g. python or ruby.")),
+
         opt_group.add_option(
             '--no-bootstrap-mrjob', dest='bootstrap_mrjob',
             action='store_false', default=None,
@@ -107,10 +125,6 @@ def add_runner_opts(opt_group):
                   " when we run this job. Use this if you've already installed"
                   " mrjob on your Hadoop cluster.")),
 
-        opt_group.add_option(
-            '--no-conf', dest='conf_path', action='store_false', default=None,
-            help="Don't load mrjob.conf even if it's available"),
-
         opt_group.add_option(
             '--no-output', dest='no_output',
             default=None, action='store_true',
@@ -122,12 +136,6 @@ def add_runner_opts(opt_group):
             'for EMR, an HDFS path for Hadoop, and a system path for local,' +
             'and must be empty'),
 
-        opt_group.add_option(
-            '--partitioner', dest='partitioner', default=None,
-            help=('Hadoop partitioner class to use to determine how mapper'
-                  ' output should be sorted and distributed to reducers. For'
-                  ' example: org.apache.hadoop.mapred.lib.HashPartitioner')),
-
         opt_group.add_option(
             '--python-archive', dest='python_archives', default=[],
             action='append',
@@ -137,22 +145,25 @@ def add_runner_opts(opt_group):
 
         opt_group.add_option(
             '--python-bin', dest='python_bin', default=None,
-            help=("Name/path of alternate python binary for mappers/reducers."
-                  " You can include arguments, e.g. --python-bin 'python"
-                  " -v'")),
+            help=("Deprecated. Name/path of alternate python binary for"
+                  " wrapper script and Python mappers/reducers. You can"
+                  " include arguments, e.g. --python-bin 'python -v'")),
 
         opt_group.add_option(
-            '-q', '--quiet', dest='quiet', default=None,
-            action='store_true',
-            help="Don't print anything to stderr"),
-
-        opt_group.add_option(
-            '-r', '--runner', dest='runner', default='local',
+            '-r', '--runner', dest='runner', default=default_runner,
             choices=('local', 'hadoop', 'emr', 'inline'),
             help=('Where to run the job: local to run locally, hadoop to run'
                   ' on your Hadoop cluster, emr to run on Amazon'
                   ' ElasticMapReduce, and inline for local debugging. Default'
-                  ' is local.')),
+                  ' is %s.' % default_runner)),
+
+        opt_group.add_option(
+            '--setup', dest='setup', action='append',
+            help=('A command to run before each mapper/reducer step in the'
+                  ' shell ("touch foo"). You may interpolate files'
+                  ' available via URL or on your local filesystem using'
+                  ' Hadoop Distributed Cache syntax (". setup.sh#"). To'
+                  ' interpolate archives, use #/: "cd foo.tar.gz#/; make')),
 
         opt_group.add_option(
             '--setup-cmd', dest='setup_cmds', action='append',
@@ -170,14 +181,16 @@ def add_runner_opts(opt_group):
                   ' These are run after setup_cmds.')),
 
         opt_group.add_option(
-            '--steps-python-bin', dest='steps_python_bin', default=None,
-            help='Name/path of alternate python binary to use to query the '
-            'job about its steps, if different from the current Python '
-            'interpreter. Rarely needed.'),
+            '--steps-interpreter', dest='steps_interpreter', default=None,
+            help=("Name/path of alternate interpreter binary to use to query"
+                  " the job about its steps, if different from --interpreter."
+                  " Rarely needed.")),
 
         opt_group.add_option(
-            '-v', '--verbose', dest='verbose', default=None,
-            action='store_true', help='print more messages to stderr'),
+            '--steps-python-bin', dest='steps_python_bin', default=None,
+            help=('Deprecated. Name/path of alternate python binary to use to'
+                  ' query the job about its steps, if different from the'
+                  ' current Python interpreter. Rarely needed.')),
     ]
 
 
@@ -207,25 +220,6 @@ def add_hadoop_emr_opts(opt_group):
             action='append', help='Argument of any type to pass to hadoop '
             'streaming. You can use --hadoop-arg multiple times.'),
 
-        opt_group.add_option(
-            '--hadoop-input-format', dest='hadoop_input_format', default=None,
-            help=('DEPRECATED: the hadoop InputFormat class used by the first'
-                  ' step of your job to read data. Custom formats must be'
-                  ' included in your hadoop streaming jar (see'
-                  ' --hadoop-streaming-jar). Current best practice is to'
-                  ' redefine HADOOP_INPUT_FORMAT or hadoop_input_format()'
-                  ' in your job.')),
-
-        opt_group.add_option(
-            '--hadoop-output-format', dest='hadoop_output_format',
-            default=None,
-            help=('DEPRECATED: the hadoop OutputFormat class used by the first'
-                  ' step of your job to read data. Custom formats must be'
-                  ' included in your hadoop streaming jar (see'
-                  ' --hadoop-streaming-jar). Current best practice is to'
-                  ' redefine HADOOP_OUTPUT_FORMAT or hadoop_output_format()'
-                  ' in your job.')),
-
         opt_group.add_option(
             '--hadoop-streaming-jar', dest='hadoop_streaming_jar',
             default=None,
@@ -239,6 +233,12 @@ def add_hadoop_emr_opts(opt_group):
             '--owner', dest='owner', default=None,
             help='custom username to use, to help us identify who ran the'
             ' job'),
+
+        opt_group.add_option(
+            '--partitioner', dest='partitioner', default=None,
+            help=('Hadoop partitioner class to use to determine how mapper'
+                  ' output should be sorted and distributed to reducers. For'
+                  ' example: org.apache.hadoop.mapred.lib.HashPartitioner')),
     ]
 
 
@@ -253,6 +253,16 @@ def add_hadoop_opts(opt_group):
             '--hdfs-scratch-dir', dest='hdfs_scratch_dir',
             default=None,
             help='Scratch space on HDFS (default is tmp/)'),
+
+        opt_group.add_option(
+            '--check-input-paths', dest='check_input_paths',
+            default=True, action='store_true',
+            help='Check input paths exist before running (the default)'),
+
+        opt_group.add_option(
+            '--no-check-input-paths', dest='check_input_paths',
+            default=True, action='store_false',
+            help='Skip the checks to ensure all input paths exist'),
     ]
 
 
@@ -266,7 +276,8 @@ def add_emr_opts(opt_group):
         opt_group.add_option(
             '--ami-version', dest='ami_version', default=None,
             help=(
-                'AMI Version to use (currently 1.0, 2.0, or latest).')),
+                'AMI Version to use (currently 1.0, 2.0, or latest, default'
+                ' latest).')),
 
         opt_group.add_option(
             '--aws-availability-zone', dest='aws_availability_zone',
@@ -406,6 +417,19 @@ def add_emr_opts(opt_group):
             help=('Local path of the hadoop streaming jar on the EMR node.'
                   ' Rarely necessary.')),
 
+        opt_group.add_option(
+            '--max-hours-idle', dest='max_hours_idle',
+            default=None, type='float',
+            help=("If we create a persistent job flow, have it automatically"
+                  " terminate itself after it's been idle this many hours.")),
+
+        opt_group.add_option(
+            '--mins-to-end-of-hour', dest='mins_to_end_of_hour',
+            default=None, type='float',
+            help=("If --max-hours-idle is set, control how close to the end"
+                  " of an EC2 billing hour the job flow can automatically"
+                  " terminate itself (default is 5 minutes).")),
+
         opt_group.add_option(
             '--no-pool-emr-job-flows', dest='pool_emr_job_flows',
             action='store_false',
@@ -504,6 +528,15 @@ def add_emr_opts(opt_group):
             '--ssh-tunnel-to-job-tracker', dest='ssh_tunnel_to_job_tracker',
             default=None, action='store_true',
             help='Open up an SSH tunnel to the Hadoop job tracker'),
+        opt_group.add_option(
+            '--visible-to-all-users', dest='visible_to_all_users',
+            default=None, action='store_true',
+            help='Whether the job flow is visible to all IAM users of the AWS'
+                 ' account associated with the job flow. If this value is set'
+                 ' to True, all IAM users of that AWS account can view and'
+                 ' (if they have the proper policy permissions set) manage'
+                 ' the job flow. If it is set to False, only the IAM user'
+                 ' that created the job flow can view and manage it.'),
     ]
 
 
