@@ -37,7 +37,7 @@ JOB_NAME_RE = re.compile(r'^(.*)\.(.*)\.(\d+)\.(\d+)\.(\d+)$')
 STEP_NAME_RE = re.compile(
     r'^(.*)\.(.*)\.(\d+)\.(\d+)\.(\d+): Step (\d+) of (\d+)$')
 
-log = logging.getLogger('mrjob.parse')
+log = logging.getLogger(__name__)
 
 
 ### URI PARSING ###
@@ -72,25 +72,38 @@ def parse_s3_uri(uri):
     """
     components = urlparse(uri)
     if (components.scheme not in ('s3', 's3n')
-        or '/' not in components.path):
+            or '/' not in components.path):
         raise ValueError('Invalid S3 URI: %s' % uri)
 
     return components.netloc, components.path[1:]
 
 
 @wraps(urlparse_buggy)
-def urlparse(*args, **kwargs):
-    """A wrapper for :py:func:`urlparse.urlparse` that handles buckets in S3
-    URIs correctly. (:py:func:`~urlparse.urlparse` does this correctly on its
-    own in Python 2.6+; this is just a patch for Python 2.5.)"""
-    components = urlparse_buggy(*args, **kwargs)
-    if components.netloc == '' and components.path.startswith('//'):
-        m = NETLOC_RE.match(components.path)
-        return ParseResult(components.scheme, m.group(1), m.group(2),
-                           components.params, components.query,
-                           components.fragment)
-    else:
-        return components
+def urlparse(urlstring, scheme='', allow_fragments=True, *args, **kwargs):
+    """A wrapper for :py:func:`urlparse.urlparse` with the following
+    differences:
+
+    * Handles buckets in S3 URIs correctly. (:py:func:`~urlparse.urlparse`
+      does this correctly sometime after 2.6.1; this is just a patch for older
+      Python versions.)
+    * Splits the fragment correctly in all URIs, not just Web-related ones.
+      This behavior was fixed in the Python 2.7.4 standard library but we have
+      to back-port it for previous versions.
+    """
+    # we're probably going to mess with at least one of these values and
+    # re-pack the whole thing before we return it.
+    # NB: urlparse_buggy()'s second argument changes names from
+    # 'default_scheme' to 'scheme' in Python 2.6, so urlparse_buggy() should
+    # be called with positional arguments.
+    (scheme, netloc, path, params, query, fragment) = (
+        urlparse_buggy(urlstring, scheme, allow_fragments, *args, **kwargs))
+    if netloc == '' and path.startswith('//'):
+        m = NETLOC_RE.match(path)
+        netloc = m.group(1)
+        path = m.group(1)
+    if allow_fragments and '#' in path and not fragment:
+        path, fragment = path.split('#', 1)
+    return ParseResult(scheme, netloc, path, params, query, fragment)
 
 
 ### OPTION PARSING ###
@@ -274,8 +287,9 @@ def find_interesting_hadoop_streaming_error(lines):
         2010-07-27 19:53:35,451 ERROR org.apache.hadoop.streaming.StreamJob (main): Error launching job , Output path already exists : Output directory s3://yourbucket/logs/2010/07/23/ already exists and is not empty
     """
     for line in lines:
-        match = _HADOOP_STREAMING_ERROR_RE.match(line) \
-                or _HADOOP_STREAMING_ERROR_RE_2.match(line)
+        match = (
+            _HADOOP_STREAMING_ERROR_RE.match(line) or
+            _HADOOP_STREAMING_ERROR_RE_2.match(line))
         if match:
             msg = match.group(1)
             if msg != 'Job not Successful!':
