@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2009-2012 Yelp and Contributors
+# Copyright 2009-2013 Yelp and Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@ import getpass
 import logging
 import os
 import os.path
+import pipes
 import pprint
 import re
 import shutil
@@ -46,8 +47,6 @@ except:
     JSONDecodeError = ValueError
 
 from mrjob.compat import add_translated_jobconf_for_hadoop_version
-from mrjob.setup import WorkingDirManager
-from mrjob.setup import parse_legacy_hash_path
 from mrjob.compat import supports_combiners_in_hadoop_streaming
 from mrjob.compat import uses_generic_jobconf
 from mrjob.conf import combine_cmds
@@ -60,6 +59,9 @@ from mrjob.conf import combine_path_lists
 from mrjob.conf import load_opts_from_mrjob_confs
 from mrjob.conf import OptionStore
 from mrjob.fs.local import LocalFilesystem
+from mrjob.setup import WorkingDirManager
+from mrjob.setup import parse_legacy_hash_path
+from mrjob.setup import parse_setup_cmd
 from mrjob.step import STEP_TYPES
 from mrjob.util import bash_wrap
 from mrjob.util import cmd_line
@@ -67,7 +69,7 @@ from mrjob.util import file_ext
 from mrjob.util import tar_and_gzip
 
 
-log = logging.getLogger('mrjob.runner')
+log = logging.getLogger(__name__)
 
 # use to detect globs and break into the part before and after the glob
 GLOB_RE = re.compile(r'^(.*?)([\[\*\?].*)$')
@@ -115,8 +117,10 @@ class RunnerOptionStore(OptionStore):
         'owner',
         'python_archives',
         'python_bin',
+        'setup',
         'setup_cmds',
         'setup_scripts',
+        'sh_bin',
         'steps_interpreter',
         'steps_python_bin',
         'upload_archives',
@@ -131,8 +135,10 @@ class RunnerOptionStore(OptionStore):
         'jobconf': combine_dicts,
         'python_archives': combine_path_lists,
         'python_bin': combine_cmds,
+        'setup': combine_lists,
         'setup_cmds': combine_lists,
         'setup_scripts': combine_path_lists,
+        'sh_bin': combine_cmds,
         'steps_interpreter': combine_cmds,
         'steps_python_bin': combine_cmds,
         'upload_archives': combine_path_lists,
@@ -163,8 +169,8 @@ class RunnerOptionStore(OptionStore):
         self.cascading_dicts.append(opts)
 
         if (len(self.cascading_dicts) > 2 and
-            all(len(d) == 0 for d in self.cascading_dicts[2:-1]) and
-            (len(conf_paths or []) > 0 or len(opts) == 0)):
+                all(len(d) == 0 for d in self.cascading_dicts[2:-1]) and
+                (len(conf_paths or []) > 0 or len(opts) == 0)):
             log.warning('No configs specified for %s runner' % alias)
 
         self.populate_values_from_cascading_dicts()
@@ -191,6 +197,7 @@ class RunnerOptionStore(OptionStore):
             'cleanup_on_failure': ['NONE'],
             'hadoop_version': '0.20',
             'owner': owner,
+            'sh_bin': ['sh'],
         })
 
     def _validate_cleanup(self):
@@ -246,6 +253,10 @@ class MRJobRunner(object):
     #: or ``'hadoop'``
     alias = None
 
+    # if this is true, when bootstrap_mrjob is true, add it through the
+    # setup script
+    BOOTSTRAP_MRJOB_IN_SETUP = True
+
     OPTION_STORE_CLASS = RunnerOptionStore
 
     ### methods to call from your batch script ###
@@ -347,15 +358,12 @@ class MRJobRunner(object):
         if self._script_path:
             self._working_dir_mgr.add('file', self._script_path)
 
-        # setup cmds and wrapper script
-        self._setup_scripts = []
-        for path in self._opts['setup_scripts']:
-            setup_script = parse_legacy_hash_path('file', path)
-            self._setup_scripts.append(setup_script)
-            self._working_dir_mgr.add(**setup_script)
+        # give this job a unique name
+        self._job_name = self._make_unique_job_name(
+            label=self._opts['label'], owner=self._opts['owner'])
 
         # we'll create the wrapper script later
-        self._wrapper_script_path = None
+        self._setup_wrapper_script_path = None
 
         # extra args to our job
         self._extra_args = list(extra_args) if extra_args else []
@@ -376,10 +384,14 @@ class MRJobRunner(object):
             self._working_dir_mgr.add(**parse_legacy_hash_path(
                 'archive', path, must_name='upload_archives'))
 
-        # set up python archives
-        self._python_archives = []
-        for path in self._opts['python_archives']:
-            self._add_python_archive(path)
+        # python_archives, setup, setup_cmds, and setup_scripts
+        # self._setup is a list of shell commands with path dicts
+        # interleaved; see mrjob.setup.parse_setup_cmds() for details
+        self._setup = self._parse_setup()
+        for cmd in self._setup:
+            for maybe_path_dict in cmd:
+                if isinstance(maybe_path_dict, dict):
+                    self._working_dir_mgr.add(**maybe_path_dict)
 
         # Where to read input from (log files, etc.)
         self._input_paths = input_paths or ['-']  # by default read from stdin
@@ -399,15 +411,11 @@ class MRJobRunner(object):
         self._hadoop_input_format = hadoop_input_format
         self._hadoop_output_format = hadoop_output_format
 
-        # give this job a unique name
-        self._job_name = self._make_unique_job_name(
-            label=self._opts['label'], owner=self._opts['owner'])
-
         # a local tmp directory that will be cleaned up when we're done
         # access/make this using self._get_local_tmp_dir()
         self._local_tmp_dir = None
 
-        # info about our steps. this is basically a cache for self._get_steps()
+        # A cache for self._get_steps(); also useful as a test hook
         self._steps = None
 
         # if this is True, we have to pipe input into the sort command
@@ -649,26 +657,6 @@ class MRJobRunner(object):
 
     ### internal utilities for implementing MRJobRunners ###
 
-    def _add_python_archive(self, path):
-        python_archive = parse_legacy_hash_path('archive', path)
-        self._working_dir_mgr.add(**python_archive)
-        self._python_archives.append(python_archive)
-
-    def _get_cmdenv(self):
-        """Get the environment variables to use inside Hadoop.
-
-        These should be `self._opts['cmdenv']` combined with python
-        archives added to :envvar:`PYTHONPATH`.
-        """
-        # on Windows, PYTHONPATH should be separated by ;, not :
-        # so LocalJobRunner and EMRJobRunner use different combiners for cmdenv
-        cmdenv_combiner = self.OPTION_STORE_CLASS.COMBINERS['cmdenv']
-        envs_to_combine = ([{'PYTHONPATH': self._working_dir_mgr.name(**a)}
-                            for a in self._python_archives] +
-                           [self._opts['cmdenv']])
-
-        return cmdenv_combiner(*envs_to_combine)
-
     def _get_local_tmp_dir(self):
         """Create a tmp directory on the local filesystem that will be
         cleaned up by self.cleanup()"""
@@ -706,8 +694,9 @@ class MRJobRunner(object):
         there are mappers and reducers for each step. Validate its
         output.
 
-        Returns output as described in :ref:`steps-format`. Results are
-        cached to avoid round trips to a subprocess.
+        Returns output as described in :ref:`steps-format`.
+
+        Results are cached, so call this as many times as you want.
         """
         if self._steps is None:
             if not self._script_path:
@@ -718,7 +707,7 @@ class MRJobRunner(object):
                 log.debug('> %s' % cmd_line(args))
                 # add . to PYTHONPATH (in case mrjob isn't actually installed)
                 env = combine_local_envs(os.environ,
-                                   {'PYTHONPATH': os.path.abspath('.')})
+                                         {'PYTHONPATH': os.path.abspath('.')})
                 steps_proc = Popen(args, stdout=PIPE, stderr=PIPE, env=env)
                 stdout, stderr = steps_proc.communicate()
 
@@ -737,13 +726,21 @@ class MRJobRunner(object):
                 for step in steps:
                     if step['type'] not in STEP_TYPES:
                         raise ValueError(
-                            'unexpected step type %r in steps %r' %
-                                         (step['type'], stdout))
+                            'unexpected step type %r in steps %r' % (
+                                step['type'], stdout))
 
                 self._steps = steps
 
         return self._steps
 
+    def _get_step(self, step_num):
+        """Get a single step (calls :py:meth:`_get_steps`)."""
+        return self._get_steps()[step_num]
+
+    def _num_steps(self):
+        """Get the number of steps (calls :py:meth:`get_steps`)."""
+        return len(self._get_steps())
+
     def _executable(self, steps=False):
         # default behavior is to always use an interpreter. local, emr, and
         # hadoop runners check for executable script paths and prepend the
@@ -761,15 +758,18 @@ class MRJobRunner(object):
             '--step-num=%d' % step_num,
             '--%s' % mrc,
         ] + self._mr_job_extra_args()
-        if self._wrapper_script_path:
-            return (
-                self._opts['python_bin'] +
-                [self._working_dir_mgr.name('file', self._wrapper_script_path)] +
-                args)
+
+        if self._setup_wrapper_script_path:
+            return (self._opts['sh_bin'] +
+                    [self._working_dir_mgr.name(
+                        'file', self._setup_wrapper_script_path)] +
+                    args)
         else:
             return args
 
-    def _substep_cmd_line(self, step, step_num, mrc):
+    def _substep_cmd_line(self, step_num, mrc):
+        step = self._get_step(step_num)
+
         if step[mrc]['type'] == 'command':
             # never wrap custom hadoop streaming commands in bash
             return step[mrc]['command'], False
@@ -787,31 +787,32 @@ class MRJobRunner(object):
             raise ValueError("Invalid %s step %d: %r" % (
                 mrc, step_num, step[mrc]))
 
-    def _render_substep(self, step, step_num, mrc):
+    def _render_substep(self, step_num, mrc):
+        step = self._get_step(step_num)
+
         if mrc in step:
-            return self._substep_cmd_line(
-                step, step_num, mrc)
+            return self._substep_cmd_line(step_num, mrc)
         else:
             if mrc == 'mapper':
                 return 'cat', False
             else:
                 return None, False
 
-    def _hadoop_streaming_commands(self, step, step_num):
+    def _hadoop_streaming_commands(self, step_num):
         version = self.get_hadoop_version()
 
         # Hadoop streaming stuff
         mapper, bash_wrap_mapper = self._render_substep(
-            step, step_num, 'mapper')
+            step_num, 'mapper')
 
         combiner, bash_wrap_combiner = self._render_substep(
-            step, step_num, 'combiner')
+            step_num, 'combiner')
 
         reducer, bash_wrap_reducer = self._render_substep(
-            step, step_num, 'reducer')
+            step_num, 'reducer')
 
         if (combiner is not None and
-            not supports_combiners_in_hadoop_streaming(version)):
+                not supports_combiners_in_hadoop_streaming(version)):
 
             # krazy hack to support combiners on hadoop <0.20
             bash_wrap_mapper = True
@@ -858,86 +859,140 @@ class MRJobRunner(object):
                 args.append(self._working_dir_mgr.name(**path_dict))
         return args
 
-    def _wrapper_script_content(self):
-        """Output a python script to the given file descriptor that runs
-        setup_cmds and setup_scripts, and then runs its arguments.
+    def _create_setup_wrapper_script(self, dest='setup-wrapper.sh'):
+        """Create the wrapper script, and write it into our local temp
+        directory (by default, to a file named wrapper.sh).
+
+        This will set ``self._setup_wrapper_script_path``, and add it to
+        ``self._working_dir_mgr``
+
+        This will do nothing if ``self._setup`` is empty or
+        this method has already been called.
+        """
+        if self._setup_wrapper_script_path:
+            return
+
+        setup = self._setup
+
+        if self._opts['bootstrap_mrjob'] and self.BOOTSTRAP_MRJOB_IN_SETUP:
+            # patch setup to add mrjob.tar.gz to PYTYHONPATH
+            mrjob_tar_gz = self._create_mrjob_tar_gz()
+            path_dict = {'type': 'archive', 'name': None, 'path': mrjob_tar_gz}
+            self._working_dir_mgr.add(**path_dict)
+            setup = [['export PYTHONPATH=', path_dict, ':$PYTHONPATH']] + setup
+
+        if not setup:
+            return
+
+        path = os.path.join(self._get_local_tmp_dir(), dest)
+        log.info('writing wrapper script to %s' % path)
+
+        contents = self._setup_wrapper_script_content(setup)
+        for line in StringIO(contents):
+            log.debug('WRAPPER: ' + line.rstrip('\r\n'))
+
+        with open(path, 'w') as f:
+            f.write(contents)
+
+        self._setup_wrapper_script_path = path
+        self._working_dir_mgr.add('file', self._setup_wrapper_script_path)
+
+    def _parse_setup(self):
+        """Helper for :py:meth:`_create_setup_wrapper_script`.
+
+        Parse the *setup* option with
+        :py:func:`mrjob.setup.parse_setup_cmd()`.
+
+        If *bootstrap_mrjob* and ``self.BOOTSTRAP_MRJOB_IN_SETUP`` are both
+        true, create mrjob.tar.gz (if it doesn't exist already) and
+        prepend a setup command that adds it to PYTHONPATH.
+
+        Also patch in the deprecated
+        options *python_archives*, *setup_cmd*, and *setup_script*
+        as setup commands.
+        """
+        setup = []
+
+        # python_archives
+        for path in self._opts['python_archives']:
+            path_dict = parse_legacy_hash_path('archive', path)
+            setup.append(['export PYTHONPATH=', path_dict, ':$PYTHONPATH'])
+
+        # setup
+        for cmd in self._opts['setup']:
+            setup.append(parse_setup_cmd(cmd))
+
+        # setup_cmds
+        for cmd in self._opts['setup_cmds']:
+            if not isinstance(cmd, basestring):
+                cmd = cmd_line(cmd)
+            setup.append([cmd])
+
+        # setup_scripts
+        for path in self._opts['setup_scripts']:
+            path_dict = parse_legacy_hash_path('file', path)
+            setup.append([path_dict])
+
+        return setup
+
+    def _setup_wrapper_script_content(self, setup, mrjob_tar_gz_name=None):
+        """Return a (Bourne) shell script that runs the setup commands and then
+        executes whatever is passed to it (this will be our mapper/reducer).
+
+        We obtain a file lock so that two copies of the setup commands
+        cannot run simultaneously on the same machine (this helps for running
+        :command:`make` on a shared source code archive, for example).
         """
         out = StringIO()
 
         def writeln(line=''):
             out.write(line + '\n')
 
-        # imports
-        writeln('from fcntl import flock, LOCK_EX, LOCK_UN')
-        writeln('from subprocess import check_call, PIPE')
-        writeln('import sys')
+        # we're always going to execute this script as an argument to
+        # sh, so there's no need to add a shebang (e.g. #!/bin/sh)
+
+        writeln('# store $PWD')
+        writeln('__mrjob_PWD=$PWD')
+        writeln('')
+
+        writeln('# obtain exclusive file lock')
+        # Basically, we're going to tie file descriptor 9 to our lockfile,
+        # use a subprocess to obtain a lock (which we somehow inherit too),
+        # and then release the lock by closing the file descriptor.
+        # File descriptors 10 and higher are used internally by the shell,
+        # so 9 is as out-of-the-way as we can get.
+        writeln('exec 9>/tmp/wrapper.lock.%s' % self._job_name)
+        # would use flock(1), but it's not always available
+        writeln("%s -c 'import fcntl; fcntl.flock(9, fcntl.LOCK_EX)'" %
+                cmd_line(self._opts['python_bin']))
         writeln()
 
-        # make lock file and lock it
-        writeln("lock_file = open('/tmp/wrapper.lock.%s', 'a')" %
-                self._job_name)
-        writeln('flock(lock_file, LOCK_EX)')
+        writeln('# setup commands')
+        for cmd in setup:
+            # reconstruct the command line, substituting $__mrjob_PWD/<name>
+            # for path dicts
+            line = ''
+            for token in cmd:
+                if isinstance(token, dict):
+                    # it's a path dictionary
+                    line += '$__mrjob_PWD/'
+                    line += pipes.quote(self._working_dir_mgr.name(**token))
+                else:
+                    # it's raw script
+                    line += token
+            writeln(line)
         writeln()
 
-        # run setup cmds
-        if self._opts['setup_cmds']:
-            writeln('# run setup cmds:')
-            for cmd in self._opts['setup_cmds']:
-                # only use the shell for strings, not for lists of arguments
-                # redir stdout to /dev/null so that it won't get confused
-                # with the mapper/reducer's output
-                writeln(
-                    "check_call(%r, shell=%r, stdout=open('/dev/null', 'w'))"
-                    % (cmd, bool(isinstance(cmd, basestring))))
-            writeln()
-
-        # run setup scripts
-        if self._setup_scripts:
-            writeln('# run setup scripts:')
-            for setup_script in self._setup_scripts:
-                writeln("check_call(%r, stdout=open('/dev/null', 'w'))" % (
-                    ['./' + self._working_dir_mgr.name(**setup_script)],))
-            writeln()
-
-        # unlock the lock file
-        writeln('flock(lock_file, LOCK_UN)')
+        writeln('# release exclusive file lock')
+        writeln('exec 9>&-')
         writeln()
 
-        # run the real script
-        writeln('# run the real mapper/reducer')
-        writeln('check_call(sys.argv[1:])')
+        writeln('# run job from the original working directory')
+        writeln('cd $__mrjob_PWD')
+        writeln('"$@"')
 
         return out.getvalue()
 
-    def _create_wrapper_script(self, dest='wrapper.py'):
-        """Create the wrapper script, and write it into our local temp
-        directory (by default, to a file named wrapper.py).
-
-        This will set self._wrapper_script_path, and add it to self._working_dir_mgr
-
-        This will do nothing if setup_cmds and setup_scripts are
-        empty, or _create_wrapper_script() has already been called.
-        """
-        if not (self._opts['setup_cmds'] or self._setup_scripts):
-            return
-
-        if self._wrapper_script_path:
-            return
-
-        path = os.path.join(self._get_local_tmp_dir(), dest)
-        log.info('writing wrapper script to %s' % path)
-
-        contents = self._wrapper_script_content()
-        for line in StringIO(contents):
-            log.debug('WRAPPER: ' + line.rstrip('\r\n'))
-
-        f = open(path, 'w')
-        f.write(contents)
-        f.close()
-
-        self._wrapper_script_path = path
-        self._working_dir_mgr.add('file', self._wrapper_script_path)
-
     def _get_input_paths(self):
         """Get the paths to input files, dumping STDIN to a local
         file if need be."""
@@ -962,12 +1017,15 @@ class MRJobRunner(object):
 
     def _create_mrjob_tar_gz(self):
         """Make a tarball of the mrjob library, without .pyc or .pyo files,
-        and return its path. This will also set self._mrjob_tar_gz_path
+        This will also set ``self._mrjob_tar_gz_path`` and return it.
+
+        Typically called from
+        :py:meth:`_create_setup_wrapper_script`.
 
         It's safe to call this method multiple times (we'll only create
         the tarball once.)
         """
-        if self._mrjob_tar_gz_path is None:
+        if not self._mrjob_tar_gz_path:
             # find mrjob library
             import mrjob
 
@@ -978,28 +1036,41 @@ class MRJobRunner(object):
 
             mrjob_dir = os.path.dirname(mrjob.__file__) or '.'
 
-            tar_gz_path = os.path.join(
-                self._get_local_tmp_dir(), 'mrjob.tar.gz')
+            tar_gz_path = os.path.join(self._get_local_tmp_dir(),
+                                       'mrjob.tar.gz')
 
             def filter_path(path):
                 filename = os.path.basename(path)
                 return not(file_ext(filename).lower() in ('.pyc', '.pyo') or
-                    # filter out emacs backup files
-                    filename.endswith('~') or
-                    # filter out emacs lock files
-                    filename.startswith('.#') or
-                    # filter out MacFuse resource forks
-                    filename.startswith('._'))
+                           # filter out emacs backup files
+                           filename.endswith('~') or
+                           # filter out emacs lock files
+                           filename.startswith('.#') or
+                           # filter out MacFuse resource forks
+                           filename.startswith('._'))
 
             log.debug('archiving %s -> %s as %s' % (
                 mrjob_dir, tar_gz_path, os.path.join('mrjob', '')))
             tar_and_gzip(
                 mrjob_dir, tar_gz_path, filter=filter_path, prefix='mrjob')
+
             self._mrjob_tar_gz_path = tar_gz_path
 
         return self._mrjob_tar_gz_path
 
-    def _hadoop_conf_args(self, step, step_num, num_steps):
+    def _jobconf_for_step(self, step_num):
+        """Get the jobconf dictionary, optionally including step-specific
+        jobconf info.
+
+        Also translate jobconfs to the current Hadoop version, if necessary.
+        """
+        step = self._get_step(step_num)
+        jobconf = combine_dicts(self._opts['jobconf'], step.get('jobconf'))
+
+        return add_translated_jobconf_for_hadoop_version(
+            jobconf, self.get_hadoop_version())
+
+    def _hadoop_args_for_step(self, step_num):
         """Build a list of extra arguments to the hadoop binary.
 
         This handles *cmdenv*, *hadoop_extra_args*, *hadoop_input_format*,
@@ -1008,12 +1079,10 @@ class MRJobRunner(object):
         This doesn't handle input, output, mappers, reducers, or uploading
         files.
         """
-        assert 0 <= step_num < num_steps
+        assert 0 <= step_num < self._num_steps()
 
         args = []
 
-        jobconf = combine_dicts(self._opts['jobconf'], step.get('jobconf'))
-
         # hadoop_extra_args
         args.extend(self._opts['hadoop_extra_args'])
 
@@ -1022,22 +1091,24 @@ class MRJobRunner(object):
 
         # translate the jobconf configuration names to match
         # the hadoop version
-        jobconf = add_translated_jobconf_for_hadoop_version(jobconf,
-                                                            version)
+        jobconf = self._jobconf_for_step(step_num)
+
         if uses_generic_jobconf(version):
             for key, value in sorted(jobconf.iteritems()):
-                args.extend(['-D', '%s=%s' % (key, value)])
+                if value is not None:
+                    args.extend(['-D', '%s=%s' % (key, value)])
         # old-style jobconf
         else:
             for key, value in sorted(jobconf.iteritems()):
-                args.extend(['-jobconf', '%s=%s' % (key, value)])
+                if value is not None:
+                    args.extend(['-jobconf', '%s=%s' % (key, value)])
 
         # partitioner
         if self._partitioner:
             args.extend(['-partitioner', self._partitioner])
 
         # cmdenv
-        for key, value in sorted(self._get_cmdenv().iteritems()):
+        for key, value in sorted(self._opts['cmdenv'].iteritems()):
             args.append('-cmdenv')
             args.append('%s=%s' % (key, value))
 
@@ -1046,7 +1117,7 @@ class MRJobRunner(object):
             args.extend(['-inputformat', self._hadoop_input_format])
 
         # hadoop_output_format
-        if (step_num == num_steps - 1 and self._hadoop_output_format):
+        if (step_num == self._num_steps() - 1 and self._hadoop_output_format):
             args.extend(['-outputformat', self._hadoop_output_format])
 
         return args
